#issue-opener {
  display: none;
}

#django-issues-modal-overlay {
  position: fixed;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  z-index: 1000;
  display: flex;
  justify-content: center;
  align-items: center;
  background-color: rgba(0, 0, 0, 0.6);
}
#django-issues-modal-overlay #django-issues-form-container {
  background-color: #AAAAAA;
  color: black;
  padding: 10px;
  overflow-y: auto;
  border: 1px #777777 solid;
  border-radius: 5px;
}
#django-issues-modal-overlay #django-issues-form-container form {
  display: flex;
  flex-direction: column;
  margin: 10px;
}
#django-issues-modal-overlay #django-issues-form-container form p {
  margin-bottom: 4px;
}
#django-issues-modal-overlay #django-issues-form-container form textarea {
  width: 97%;
  margin: 0;
}
#django-issues-modal-overlay #django-issues-form-container form select {
  width: 100%;
  height: 40px;
}
#django-issues-modal-overlay #django-issues-form-container form input[type=text] {
  line-height: 30px;
  margin: 0;
  width: 97%;
}
#django-issues-modal-overlay #django-issues-form-container form #screenshot-preview-container {
  width: 100%;
  margin: 2px 0;
  height: 200px;
  transition: opacity 0.3s, visibility 0.3s;
  visibility: hidden;
  opacity: 0;
}
#django-issues-modal-overlay #django-issues-form-container form #screenshot-preview-container #screenshot-preview-img {
  max-height: 100%;
  max-width: 100%;
}
#django-issues-modal-overlay #django-issues-form-container form button {
  background: #417690;
  color: white;
  padding: 5px 15px;
  margin-top: 5px;
  height: 2.1875rem;
  line-height: 0.9375rem;
  border: none;
  border-radius: 4px;
  cursor: pointer;
  transition: background 0.15s;
}

/*# sourceMappingURL=issues.css.map */
